<?php

/**
 * Execute a SQL query and return the result as an array of objects or arrays.
 *
 * @param string $query SQL query
 * @param bool $useObject Use objects if true, arrays if false
 * @return array Result set
 */
function qSQL($query, $useObject = true)
{
    global $pdo;
    $stmt = $pdo->query($query);

    $return = [];
    if ($stmt) {
        $return = $useObject ? $stmt->fetchAll(PDO::FETCH_OBJ) : $stmt->fetchAll(PDO::FETCH_ASSOC);
    }

    return $return;
}

/**
 * Get a single result from a query by specifying row and column.
 *
 * @param PDOStatement $res PDOStatement result from a query
 * @param int $row Row number
 * @param int $col Column number
 * @return mixed Result value
 */
function pdo_result($res, $row = 0, $col = 0)
{
    global $pdo;
    $res->setFetchMode(PDO::FETCH_NUM);
    $data = $res->fetchAll();

    if (isset($data[$row][$col])) {
        return $data[$row][$col];
    }

    return false;
}

/**
 * Count the number of rows in a table.
 *
 * @param string $tbname Table name
 * @return int Number of rows
 */
function counting($tbname)
{
    global $pdo;
    $query = "SELECT COUNT(1) FROM " . $tbname;
    $stmt = $pdo->query($query);
    $count = $stmt->fetchColumn();
    return $count;
}

/**
 * Get the size of a table.
 *
 * @param string $tbname Table name
 * @return float Size of the table in MB
 */
function tablesize($tbname)
{
    global $dbs, $pdo, $dbname;
    if ($dbs == '1') {
        $query = "SELECT table_name, (data_length + index_length) / 1024 / 1024 AS tb_size
              FROM information_schema.TABLES
              WHERE table_schema='$dbname' AND table_name='$tbname'";
    } elseif ($dbs == '2') {
        // SQLite query to get table size
        $query = "SELECT name, (length(sql) / 1024 / 1024) AS size_mb
    FROM sqlite_master
    WHERE type='table' AND name='$tbname'";
    }
    $stmt = $pdo->query($query);
    $data = $stmt->fetch(PDO::FETCH_ASSOC);

    return $data['tb_size'];
}

/**
 * Get a row by ID from a table.
 *
 * @param string $tbname Table name
 * @param int $id ID
 * @return mixed Result row or false if not found
 */
function getById($tbname, $id)
{
    global $pdo;
    $getByField = getByField($tbname);
    //echo $getByField['name']."/";
    $getByField = $getByField['Field'];
    //echo $getByField ; exit(1);
    $query = "SELECT * FROM {$tbname} WHERE {$getByField} = :id";
    $stmt = $pdo->prepare($query);
    $stmt->bindParam(':id', $id, PDO::PARAM_INT);
    $stmt->execute();

    return $stmt->fetch(PDO::FETCH_ASSOC);
}

/**
 * Get the first field from a table.
 *
 * @param string $tbname Table name
 * @return mixed Result field or false if not found
 */
function getByField($tbname)
{
    global $pdo, $dbs;
    if ($dbs == '1') {
        $query = "SHOW COLUMNS FROM " . $tbname;
    } elseif ($dbs == '2') {
        $query = "SELECT
                    `name` AS 'Field',
                    `type` AS 'Type', 
                    CASE WHEN `notnull` = 1 THEN 'YES' ELSE 'NO' END AS 'Null',
                    NULL AS 'Key',
                    `dflt_value` AS 'Default',
                    CASE WHEN `pk` = 1 THEN 'auto_increment' ELSE '' END AS 'Extra'
                FROM
                    pragma_table_info('{$tbname}');
                ";
    }
    //echo $query; exit(1);
    $stmt = $pdo->query($query);

    return $stmt->fetch(PDO::FETCH_ASSOC);
}

/**
 * Get all rows from a table.
 *
 * @param string $tbname Table name
 * @return array Result set
 */
function getAll($tbname)
{
    global $pdo;
    $query = "SELECT * FROM " . $tbname;
    return qSQL($query);
}

/**
 * Execute a SELECT query for generating an HTML select element.
 *
 * @param string $tbname Table name
 * @param string $where WHERE clause
 * @param string $operator Comparison operator
 * @param string $zero_value Value for the "Please select" option
 * @param string $key Column to use as the option value
 * @param string $value Column to use as the option text
 * @param string $id Selected ID
 * @return string HTML select element
 */
function queryToSelect($tbname, $where, $operator, $zero_value, $key, $value, $id)
{
    global $pdo;
    $ul = '<option value="' . $zero_value . '">Please select</option>';

    $query = "SELECT * FROM '$tbname' WHERE '$where' '$operator' '$zero_value'";
    $result = qSQL($query);

    foreach ($result as $row) {
        $ul .= '<option value="' . $row[$key] . '" ';
        $ul .= $id == $row[$key] ? "selected" : "";
        $ul .= '>' . $row[$value] . '</option>';
    }

    return $ul;
}
/**
 * @param string $tbname Table name (not null)
 * @param array $requestData Request data (not null)
 * @param array|null $columns Columns (null)
 * @return array Result data for DataTables
 */
function getDataTable($tbname, $requestData = [], $columns = null)
{
    global $pdo;

    if (!empty($requestData)) {
        $sql_no = "SELECT COUNT(*) as total FROM $tbname WHERE 1";

        // Count all
        $stmt_no = $pdo->query($sql_no);
        $totalData = pdo_result($stmt_no, 0, 0);
        $totalFiltered = $totalData;

        $sWhere = "";
        if (!empty($columns)) {
            if ($requestData['search']['value'] != "") {
                $sWhere = " AND (";
                $c = count($columns);
                for ($i = 0; $i < $c; $i++) {
                    if ($i == 3 && $tbname == 'genusers') {
                        $sWhere .= $columns[$i] . " LIKE '" . encodeTEN($requestData['search']['value']) . "%' OR ";
                    } else {
                        $sWhere .= $columns[$i] . " LIKE '" . $requestData['search']['value'] . "%' OR ";
                    }
                }
                $sWhere = substr_replace($sWhere, "", -3);
                $sWhere .= ')';

                // Count all with search
                $sql_no .= $sWhere;
                $stmt_data = $pdo->query($sql_no);
                $totalFiltered = pdo_result($stmt_data, 0, 0);
            }
        }

        // Data to be fetched
        $sql_data = ($tbname == 'genusers')
            ? "SELECT name, email, password, fname, image, files, role, id FROM $tbname WHERE 1"
            : "SELECT * FROM $tbname WHERE 1";
        $sql_data .= $sWhere;
        $sql_data .= " ORDER BY " . $columns[$requestData['order'][0]['column']] . " " . $requestData['order'][0]['dir'] . " LIMIT " . $requestData['start'] . " ," . $requestData['length'];
        $data = qSQL($sql_data);

        $result = [
            "draw"            => intval($requestData['draw']),
            "recordsTotal"    => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data"            => $data,
        ];
    } else {
        $result = ['status' => false, 'message' => 'No requestData provided'];
    }
    $pdo=NULL;
    return objectToArray($result);
}

if (!function_exists('objectToArray')) {
    function objectToArray($object)
    {
        if (is_object($object) || is_array($object)) {
            $array = array();
            foreach ($object as $key => $value) {
                $array[$key] = objectToArray($value);
            }
            return $array;
        }
        return $object;
    }
}
